import _extends from 'babel-runtime/helpers/extends';
import Icon from '../icon';
import ListItemMixin from '../mixins/list-item';
import classNames from '../_utils/classnames';
import { isDef } from '../utils/commonUtil';
import './index.less';

var ListItem = {
    functional: true,
    name: "ListItem",
    mixins: [ListItemMixin],
    onClick: function onClick(event) {
        // emit(ctx, 'click', event);
        // functionalRoute(ctx);
    },
    render: function render(h, context) {
        var props = context.props,
            scopedSlots = context.scopedSlots,
            data = context.data;
        var icon = props.icon,
            size = props.size,
            title = props.title,
            label = props.label,
            value = props.value,
            isLink = props.isLink,
            arrowDirection = props.arrowDirection,
            labelNumber = props.labelNumber;


        var showTitle = scopedSlots.title || isDef(title);
        var showValue = scopedSlots['default'] || isDef(value);
        var showLabel = scopedSlots.label || isDef(label);

        var Label = showLabel && h(
            'div',
            { 'class': {
                    'sn-list-item-label': true,
                    '': props.labelClass
                } },
            [scopedSlots.label ? scopedSlots.label() : label]
        );

        var Title = showTitle && h(
            'div',
            {
                'class': {
                    'sn-list-item-title': true,
                    '': props.titleClass
                },
                style: props.titleStyle
            },
            [scopedSlots.title ? scopedSlots.title() : h('span', [title]), Label]
        );

        var Value = showValue && h(
            'div',
            { 'class': {
                    'sn-list-item-value': true,
                    alone: !scopedSlots.title && !title,
                    '': props.valueClass
                } },
            [scopedSlots['default'] ? scopedSlots['default']() : h('span', [value])]
        );

        var LeftIcon = scopedSlots.icon ? scopedSlots.icon() : icon && h(Icon, { 'class': 'sn-list-item-left-icon', attrs: { type: icon }
        });

        var rightIconSlot = scopedSlots['right-icon'];
        var RightIcon = rightIconSlot ? rightIconSlot() : isLink && h(Icon, {
            'class': 'sn-list-item-right-icon',
            attrs: { type: arrowDirection ? arrowDirection : 'right'
            }
        });

        var clickable = isLink || props.clickable;
        var classes = classNames({
            'sn-list-item': true,
            'sn-list-item-clickable': clickable,
            'sn-list-item-center': props.center,
            'sn-list-item-required': props.required,
            'sn-list-item-borderless': !props.border
        });

        if (size) {
            classes[size] = size;
        }

        var domProps = _extends({}, data, {
            attrs: {
                role: clickable ? 'button' : null,
                tabindex: clickable ? 0 : null
            },
            'class': classes
        });

        return h(
            'div',
            domProps,
            [LeftIcon, Title, Value, RightIcon, scopedSlots.extra && scopedSlots.extra()]
        );
    }
};

export default ListItem;