import _extends from 'babel-runtime/helpers/extends';
import defaultRequest from './request';
export default {
    name: 'AjaxUploader',
    props: {
        multiple: {
            type: Boolean,
            'default': function _default() {
                return true;
            } // or false
        },
        disabled: {
            type: Boolean,
            'default': function _default() {
                return false;
            }
        },
        action: {
            type: String,
            'default': function _default() {
                return '';
            }
        },
        beforeUpload: Function,
        customRequest: Function
    },
    data: function data() {
        return {
            index: 0,
            now: +new Date(),
            uid: this.getUid(),
            reqs: function reqs() {
                return [];
            }
        };
    },
    beforeDestroy: function beforeDestroy() {
        this.abort();
    },

    methods: {
        getUid: function getUid() {
            return 'vc-upload-' + this.now + '-' + ++this.index;
        },
        onClick: function onClick() {
            var el = this.$refs.fileInputRef;
            if (!el) {
                return;
            }
            el.click();
        },
        onChange: function onChange(e) {
            var files = e.target.files;
            if (files) {
                this.uploadfiles(files);
                this.$refs.fileInputRef.value = null;
            }
        },
        uploadfiles: function uploadfiles(files) {
            var _this = this;

            var fileList = Array.prototype.slice.call(files);
            fileList.forEach(function (file) {
                file.uid = _this.getUid();
                _this.upload(file, fileList);
            });
        },
        upload: function upload(file, fileList) {
            var _this2 = this;

            if (!this.beforeUpload) {
                this.post(file);
                return;
            }
            var before = this.beforeUpload(file, fileList);
            if (before && before.then) {
                before.then(function (processFile) {
                    console && console.log(processFile);
                    _this2.post(file);
                })['catch'](function (e) {
                    console && console.log(e); // eslint-disable-line
                });
            } else if (before !== false) {
                this.post(file);
            }
        },
        post: function post(file) {
            var _this3 = this;

            var uid = file.uid;
            var action = this.$props.action;

            var request = this.customRequest || defaultRequest;
            var options = {
                action: action,
                filename: this.name,
                file: file,
                headers: this.headers,
                onProgress: function onProgress(e) {
                    _this3.$emit('progress', e, file);
                },
                onSuccess: function onSuccess(ret, xhr) {
                    if (file.currentChunk >= file.chunks) {
                        delete _this3.reqs[uid];
                    }
                    file.options = options;
                    _this3.$emit('success', file, ret, xhr);
                },
                onError: function onError(err, ret) {
                    console.log("directUpload failed. fileName: ");
                    delete _this3.reqs[uid];
                    _this3.$emit('error', err, ret, file);
                }
            };
            this.$emit('start', file);
            this.reqs[uid] = request(options);
        },
        abort: function abort(file) {
            var reqs = this.reqs;

            Object.keys(reqs).forEach(function (uid) {
                if (reqs[uid]) {
                    reqs[uid].abort();
                }
                delete reqs[uid];
            });
        }
    },
    render: function render() {
        var h = arguments[0];
        var _$props = this.$props,
            multiple = _$props.multiple,
            disabled = _$props.disabled;

        var events = disabled ? {} : {
            click: this.onClick
        };
        var btnProps = {
            on: _extends({}, this.$listeners, events)
        };
        return h(
            'span',
            btnProps,
            [h('input', {
                attrs: {
                    type: 'file',

                    multiple: multiple
                },
                key: this.uid,
                ref: 'fileInputRef',
                style: { display: 'none' }, on: {
                    'change': this.onChange
                }
            }), this.$slots['default']]
        );
    }
};