'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _carousel = require('../carousel');

var _carousel2 = _interopRequireDefault(_carousel);

var _pickerView = require('../picker-view');

var _pickerView2 = _interopRequireDefault(_pickerView);

var _icon = require('../icon');

var _icon2 = _interopRequireDefault(_icon);

var _util = require('../calendar/util');

var _dateUtil = require('../utils/dateUtil');

var _numberUtil = require('../utils/numberUtil');

var _event = require('../_utils/dom/event');

var _raf = require('../_utils/dom/raf');

require('./index.less');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var CarouselItem = _carousel2['default'].Item;

exports['default'] = {
    name: "RangePickerView",
    props: {
        value: {
            type: Array
        },
        monthSwiperMode: { 'default': "vertical" //月份滑动的方式，左右还是上下滑动
        } },
    data: function data() {
        return {
            height: '',
            beginDate: '',
            endDate: '',
            beginDateSelect: true,
            endDateSelect: false,
            selectedDate: [],
            selectedDay: '',
            weekList: ["日", "一", "二", "三", "四", "五", "六"],
            yearAndMonthList: [],
            showCalender: true,
            showMonthSelect: false,
            year: 0,
            month: 0,
            currentValue: '',
            currentDate: '',
            swiperList: [],
            swiperItemIndex: 0
        };
    },

    watch: {
        value: function value(val) {
            this.currentValue = val;
        },
        selectedDay: function selectedDay(val, oldVal) {
            var beginDateSelect = this.beginDateSelect,
                endDateSelect = this.endDateSelect;

            if (beginDateSelect) {
                this.beginDate = val;
            } else if (endDateSelect) {
                this.endDate = val;
            }
        }
    },
    created: function created() {
        this.currentValue = this.value || [(0, _dateUtil.dateFormat)(new Date(), 'YYYY/MM/DD'), (0, _dateUtil.dateFormat)(new Date(), 'YYYY/MM/DD')];

        if (this.currentValue instanceof Array) {
            if (this.currentValue.length === 1) {
                var beginDate = this.currentValue[0];
                if (isNaN(beginDate) && !isNaN(Date.parse(beginDate))) {
                    beginDate = (0, _dateUtil.dateFormat)(beginDate, 'YYYY/MM/DD');
                } else {
                    beginDate = (0, _dateUtil.dateFormat)(new Date(), 'YYYY/MM/DD');
                }

                this.beginDate = beginDate;
                this.endDate = beginDate;
                this.selectedDay = beginDate;
            } else if (this.currentValue.length === 2) {
                var _beginDate = this.currentValue[0];
                var endDate = this.currentValue[1];
                if (_beginDate && isNaN(_beginDate) && !isNaN(Date.parse(_beginDate))) {
                    _beginDate = (0, _dateUtil.dateFormat)(_beginDate, 'YYYY/MM/DD');
                } else {
                    _beginDate = (0, _dateUtil.dateFormat)(new Date(), 'YYYY/MM/DD');
                }
                if (endDate && isNaN(endDate) && !isNaN(Date.parse(endDate))) {
                    endDate = (0, _dateUtil.dateFormat)(endDate, 'YYYY/MM/DD');
                } else {
                    endDate = (0, _dateUtil.dateFormat)(new Date(), 'YYYY/MM/DD');
                }

                this.beginDate = _beginDate;
                this.endDate = endDate;
                this.selectedDay = this.beginDate;
            }
        }
        var date = new Date(this.beginDate);
        var year = date.getFullYear(),
            month = date.getMonth();
        this.render(year, month);
        this.renderYearAndMonth(year, month);
    },
    mounted: function mounted() {
        /* 动态计算swiper高度 */
        this.getHeight();
    },

    methods: {
        getHeight: function getHeight() {
            var height = this.$refs.month.offsetHeight;
            console.log('month == ', height);
            var windowWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
            this.height = windowWidth < 480 ? height + 'px' : '330px';
        },
        render: function render(year, month) {
            var force = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

            this.year = year;
            this.month = month;

            /* 注入滑动数组 */
            this.renderCarousel(year, month, this.selectedDay);
        },

        /* 渲染单页两个月数据 */
        renderCarousel: function renderCarousel(year, month, value) {
            var beginDate = this.beginDate,
                endDate = this.endDate;

            var preSwiper = void 0,
                curSwiper = void 0,
                nextSwiper = void 0;
            /* 生成总共需要的3个月数据 */
            var prevTime = (0, _util.getPrevTime)(year, month);
            var nextTime = (0, _util.getNextTime)(year, month);
            var prevMonth = (0, _util.getDays)((0, _extends3['default'])({}, prevTime, {
                value: value,
                rangeBegin: new Date(beginDate),
                rangeEnd: new Date(endDate),
                returnSixRows: true
            }));
            var curMonth = (0, _util.getDays)({ year: year, month: month, value: value, returnSixRows: true });
            var nextMonth = (0, _util.getDays)((0, _extends3['default'])({}, nextTime, { value: value, returnSixRows: true }));

            /* 填充第一页两个月数据 */
            curSwiper = curMonth;
            /* 填充下一页两个月数据 */
            nextSwiper = nextMonth;
            /* 填充上一页两个月数据 */
            preSwiper = prevMonth;
            this.swiperList = [];
            this.swiperList = [curSwiper, nextSwiper, preSwiper];
        },

        /* 根据指定日期生成上下20年年月数据 */
        renderYearAndMonth: function renderYearAndMonth(year, month) {
            var _date = new Date();
            var _year = _date.getFullYear();
            var yearList = [],
                monthList = [];
            // 加载前后20年数据
            for (var i = _year - 20; i <= _year + 20; i++) {
                yearList.push('' + i);
            }
            // 加载12个月份
            for (var _i = 1; _i <= 12; _i++) {
                var _month2 = (0, _numberUtil.zeroComplete)(_i);
                monthList.push('' + _month2);
            }

            this.yearAndMonthList = [{
                values: yearList,
                defaultIndex: 0
            }, {
                values: monthList,
                defaultIndex: 1
            }];
            this.selectedDate = ['' + year, '' + (0, _numberUtil.zeroComplete)(month + 1)];
        },

        /* 上一页 */
        prev: function prev() {
            if (!this.minDate) {
                var swiperItemIndex = this.swiperItemIndex;
                var goIndex = swiperItemIndex - 1 >= 0 ? swiperItemIndex - 1 : 2;
                this.$refs.carousel.swipeTo(goIndex);
            }
        },

        /* 下一页 */
        next: function next() {
            if (!this.maxDate) {
                var swiperItemIndex = this.swiperItemIndex;
                var goIndex = swiperItemIndex + 1 <= 2 ? swiperItemIndex + 1 : 0;
                this.$refs.carousel.swipeTo(goIndex);
            }
        },
        go: function go(year, month) {
            this.render(year, month, true);
        },

        /* 选择日期 */
        select: function select(event, k1, k2, data) {
            (0, _event.preventDefault)(event, true);
            var index = this.swiperItemIndex;
            var month = this.swiperList[index];
            var days = month.days;
            for (var row = 0; row < days.length; row++) {
                for (var day = 0; day < days[row].length; day++) {
                    if (row === k1 && day === k2) {
                        days[k1][k2].isCurrent = true;
                    } else {
                        days[row][day].isCurrent = false;
                    }
                }
            }

            this.selectedDay = data.formatedDate;
            /* 如果当前选择的为上一月日期或者是下一月日期，重绘日历 */
            if (data.isLastMonth || data.isNextMonth) {
                var date = new Date(data.formatedDate);
                this.swiperItemIndex = 0;
                this.$refs.carousel.swipeTo(0);
                this.render(date.getFullYear(), date.getMonth());
            }
        },

        /* 设置选中的天 */
        setSelectedDay: function setSelectedDay() {
            var currentDate = new Date(this.selectedDay);
            var day = currentDate.getDate();
            /* 当前月最大日期 */
            var selectedMonthMaxDate = new Date(this.year, this.month + 1, 0).getDate();
            var selectedDay = (0, _dateUtil.dateFormat)(new Date(this.year, this.month, day < selectedMonthMaxDate ? day : selectedMonthMaxDate), 'YYYY/MM/DD');
            this.selectedDay = selectedDay;
            this.currentDate = selectedDay;

            for (var t = 0; t < this.swiperList.length; t++) {
                var month = this.swiperList[t];
                var days = month.days;
                for (var row = 0; row < days.length; row++) {
                    for (var _day = 0; _day < days[row].length; _day++) {
                        if (days[row][_day].formatedDate === selectedDay) {
                            this.swiperList[t].days[row][_day].isCurrent = true;
                        } else {
                            this.swiperList[t].days[row][_day].isCurrent = false;
                        }
                    }
                }
            }
        },

        /* 确认日期 */
        submit: function submit(event) {
            (0, _event.preventDefault)(event, true);
            var beginDate = this.beginDate,
                endDate = this.endDate;

            if (!beginDate || beginDate === '') {
                this.$toast('请选择起始日期');
                return;
            }
            if (!endDate || endDate === '') {
                this.$toast('请选择结束日期');
                return;
            }
            if (new Date(beginDate).getTime() > new Date(endDate).getTime()) {
                this.$toast({
                    position: 'bottom',
                    message: '起始日期不能大于结束日期'
                });
                return;
            }
            this.currentValue = [beginDate, endDate];

            this.$emit('change', this.currentValue);
        },

        /* 日历onChange回调 */
        onCalendarChange: function onCalendarChange(currentIndex) {
            var _this = this;

            // console.log('onCalendarChange currentIndex === ', currentIndex);
            // console.log('onCalendarChange swiperItemIndex === ', this.swiperItemIndex);
            /* 滑动结束后执行选中滑动前选中的天 */
            if (currentIndex !== this.swiperItemIndex) {
                this.$nextTick(function () {
                    (0, _raf.doubleRaf)(function () {
                        var _year = _this.year,
                            _month = _this.month,
                            _currentValue = _this.selectedDay;

                        _this.year = _this.swiperList[currentIndex].year;
                        _this.month = _this.swiperList[currentIndex].month;
                        _this.selectedDate = ['' + _this.year, '' + (0, _numberUtil.zeroComplete)(_this.month + 1)];
                        var prevTime = (0, _util.getPrevTime)(_year, _month);
                        var prev2Time = (0, _util.getPrevTime)(prevTime.year, prevTime.month);
                        var nextTime = (0, _util.getNextTime)(_year, _month);
                        var next2Time = (0, _util.getNextTime)(nextTime.year, nextTime.month);

                        var prev2Month = (0, _util.getDays)((0, _extends3['default'])({}, prev2Time, { value: _currentValue, returnSixRows: true }));
                        var next2Month = (0, _util.getDays)((0, _extends3['default'])({}, next2Time, { value: _currentValue, returnSixRows: true }));

                        /* 当前页为第一页 */
                        if (_this.swiperItemIndex === 0) {
                            /* 下滑，进入第二页; 开始顺序 => [cur, next, prev], 结束顺序 => [cur, next, next2] */
                            if (currentIndex === 1) {
                                _this.$set(_this.swiperList, 2, next2Month);
                            } else {
                                /* 上滑，进入第三页; 开始顺序 => [cur, next, prev], 结束顺序 => [cur, prev2, prev] */
                                _this.$set(_this.swiperList, 1, prev2Month);
                            }
                        }
                        /* 当前页为第二页 */
                        else if (_this.swiperItemIndex === 1) {
                                /* 下滑，进入第三页; 开始顺序 => [prev, cur, next], 结束顺序 => [next2, cur, next] */
                                if (currentIndex === 2) {
                                    _this.$set(_this.swiperList, 0, next2Month);
                                } else {
                                    /* 上滑，进入第一页; 开始顺序 => [prev, cur, next], 结束顺序 => [prev, cur, prev2] */
                                    _this.$set(_this.swiperList, 2, prev2Month);
                                }
                            }
                            /* 当前页为第三页 */
                            else if (_this.swiperItemIndex === 2) {
                                    /* 下滑，进入第一页; 开始顺序 => [next, prev, cur], 结束顺序 => [next, next2, cur] */
                                    if (currentIndex === 0) {
                                        _this.$set(_this.swiperList, 1, next2Month);
                                    } else {
                                        /* 上滑，进入第二页; 开始顺序 => [next, prev, cur], 结束顺序 => [prev2, prev, cur] */
                                        _this.$set(_this.swiperList, 0, prev2Month);
                                    }
                                }
                        _this.swiperItemIndex = currentIndex;

                        _this.setSelectedDay();
                    });
                });
            }
        },

        /* 年月onChange回调 */
        onYearAndMonthChange: function onYearAndMonthChange(value) {
            console.log('onYearAndMonthChange = ', value);
            this.selectedDate = value;
            if (value && value instanceof Array) {
                var year = parseInt(value[0]);
                var month = parseInt(value[1]);
                var selectedMonthMaxDate = new Date(year, month, 0).getDate();
                if (this.beginDateSelect) {
                    var day = new Date(this.beginDate).getDate();
                    this.beginDate = (0, _dateUtil.dateFormat)(new Date(year, month - 1, day < selectedMonthMaxDate ? day : selectedMonthMaxDate), 'YYYY/MM/DD');
                } else if (this.endDateSelect) {
                    var _day2 = new Date(this.endDate).getDate();
                    this.endDate = (0, _dateUtil.dateFormat)(new Date(year, month - 1, _day2 < selectedMonthMaxDate ? _day2 : selectedMonthMaxDate), 'YYYY/MM/DD');
                }
                // this.selectedDay = selectedDay;
            }
        },

        /* 切换视图 */
        onViewChange: function onViewChange(type) {
            var _date = void 0;
            if (type === "begin") {
                _date = this.beginDate;
                this.endDateSelect = false;

                if (this.beginDateSelect) {
                    if (this.showCalender) {
                        this.showCalender = false;
                        this.showMonthSelect = true;
                        this.$refs.datePicker.setValues(this.selectedDate);
                    } else {
                        this.showCalender = true;
                        this.showMonthSelect = false;
                    }
                } else {
                    this.beginDateSelect = true;
                    this.showCalender = true;
                    this.showMonthSelect = false;
                }
            } else if (type === "end") {
                _date = this.endDate;
                this.beginDateSelect = false;

                if (this.endDateSelect) {
                    if (this.showCalender) {
                        this.showCalender = false;
                        this.showMonthSelect = true;
                        this.$refs.datePicker.setValues(this.selectedDate);
                    } else {
                        this.showCalender = true;
                        this.showMonthSelect = false;
                    }
                } else {
                    this.endDateSelect = true;
                    this.showCalender = true;
                    this.showMonthSelect = false;
                }
            }

            var date = _date ? new Date(_date) : new Date();
            this.selectedDay = _date;
            this.selectedDate = ['' + date.getFullYear(), '' + (0, _numberUtil.zeroComplete)(date.getMonth() + 1)];
            this.render(date.getFullYear(), date.getMonth());
        },
        valueFormat: function valueFormat(value) {
            return (0, _dateUtil.dateFormat)(value, 'YYYY年MM月DD日');
        }
    },
    render: function render() {
        var _this2 = this;

        var h = arguments[0];
        var mode = this.mode,
            showMonthSelect = this.showMonthSelect,
            beginDateSelect = this.beginDateSelect,
            endDateSelect = this.endDateSelect,
            beginDate = this.beginDate,
            endDate = this.endDate,
            showCalender = this.showCalender,
            weekList = this.weekList,
            height = this.height,
            swiperList = this.swiperList,
            yearAndMonthList = this.yearAndMonthList;


        return h(
            'div',
            { 'class': 'sn-range-picker' },
            [h(
                'div',
                { 'class': 'sn-range-picker-header' },
                [h(
                    'div',
                    { 'class': 'sn-range-picker-tabs range-date' },
                    [h(
                        'div',
                        {
                            'class': {
                                "sn-range-picker-tabs-item": true,
                                "highlight": true,
                                active: beginDateSelect
                            },
                            on: {
                                'click': function click() {
                                    return _this2.onViewChange('begin');
                                }
                            }
                        },
                        [beginDate ? this.valueFormat(beginDate) : '请选择起始日期']
                    ), h(
                        'div',
                        { 'class': 'sn-range-picker-tabs-separator' },
                        [h(_icon2['default'], {
                            attrs: { type: 'minus' }
                        })]
                    ), h(
                        'div',
                        {
                            'class': {
                                "sn-range-picker-tabs-item": true,
                                "highlight": true,
                                active: endDateSelect
                            },
                            on: {
                                'click': function click() {
                                    return _this2.onViewChange('end');
                                }
                            }
                        },
                        [endDate ? this.valueFormat(endDate) : '请选择结束日期']
                    )]
                ), h(
                    'div',
                    { 'class': 'sn-range-picker-btn', on: {
                            'click': this.submit
                        }
                    },
                    ['\u786E\u5B9A']
                )]
            ), h(
                'div',
                { 'class': 'sn-range-picker-weekday', style: { visibility: showCalender ? 'visible' : 'hidden' } },
                [weekList.map(function (week) {
                    return h(
                        'div',
                        { 'class': 'sn-range-picker-weekday-item' },
                        [h('div', [week])]
                    );
                })]
            ), h(
                'div',
                { 'class': 'sn-range-picker-pane', style: { height: height } },
                [h(
                    _carousel2['default'],
                    {
                        ref: 'carousel',
                        attrs: { 'show-indicators': false,

                            loop: true,
                            vertical: true
                        },
                        on: {
                            'change': this.onCalendarChange
                        },
                        directives: [{
                            name: 'show',
                            value: showCalender
                        }],

                        style: { height: height }
                    },
                    [swiperList.map(function (month, k0) {
                        return h(
                            CarouselItem,
                            { key: k0, style: { width: '100%' } },
                            [h(
                                'div',
                                { 'class': 'sn-range-picker-pane-month', ref: 'month' },
                                [month.days.map(function (row, k1) {
                                    return h(
                                        'div',
                                        { 'class': 'month-row' },
                                        [row.map(function (item, k2) {
                                            return h(
                                                'div',
                                                { 'class': 'day' },
                                                [h(
                                                    'div',
                                                    {
                                                        'class': {
                                                            "day-pack": true,
                                                            "last-month": item.isLastMonth,
                                                            "next-month": item.isNextMonth
                                                        },
                                                        on: {
                                                            'click': function click(event) {
                                                                return _this2.select(event, k1, k2, item);
                                                            }
                                                        }
                                                    },
                                                    [h(
                                                        'div',
                                                        { 'class': {
                                                                today: item.isToday,
                                                                active: item.isCurrent,
                                                                "in-range": item.isInRange
                                                            } },
                                                        [h(
                                                            'div',
                                                            { 'class': 'day-pack-cell' },
                                                            [h(
                                                                'div',
                                                                { 'class': 'header' },
                                                                [item.isToday ? h('span', ['\u4ECA\u5929']) : null]
                                                            ), h(
                                                                'div',
                                                                {
                                                                    'class': {
                                                                        "middle": true,
                                                                        isHoliday: item.isHoliday,
                                                                        hidden: item.dayHide
                                                                    }
                                                                },
                                                                [item.day]
                                                            ), h(
                                                                'div',
                                                                { 'class': 'footer' },
                                                                [item.isCurrent ? beginDateSelect ? '起' : '止' : null]
                                                            )]
                                                        )]
                                                    )]
                                                )]
                                            );
                                        })]
                                    );
                                })]
                            )]
                        );
                    })]
                ), h(
                    'div',
                    {
                        'class': {
                            "date-select": true,
                            "single-col": mode === "year" || mode === "month"
                        },
                        directives: [{
                            name: 'show',
                            value: showMonthSelect
                        }],

                        style: { height: height }
                    },
                    [h(_pickerView2['default'], {
                        ref: 'datePicker',
                        attrs: { visibleItemCount: 7,
                            columns: yearAndMonthList
                        },
                        on: {
                            'change': this.onYearAndMonthChange
                        }
                    })]
                )]
            )]
        );
    }
};
module.exports = exports.default;